#				SSBA 2.0F
#
# Makefile : andrew.mk
#
##############################################################################
SHELL=/bin/sh

.PRECIOUS:	run
.IGNORE:	

PRINTER	=	lp
PRINTOPT=	-l66
PAGER	=	more

NAME	=	andrew
README	=	${NAME}.doc ${NAME}.files
MAKEFILE=	${NAME}.mk
CONFIG	=	${NAME}.cf
RUN	=	${NAME}.run
JOU	=	${NAME}.jou
LOG	=	${NAME}.log
AWK	=	${NAME}.awk
RES	=	${NAME}.res
KILL	=	${NAME}.kill
TAR	=	${NAME}.tar
SHAR 	=	${NAME}.shar
BANNER 	=	${NAME}.ban
SSBALOG	=	../results/ssba.log
SSBASIZE=	../results/ssba.size
TRACE	=	1>>$(JOU) 2>&1
NUL	=	1>/dev/null 2>&1
DATA	=	data
ARCHIVES=	$(TAR) $(SHAR)
VOLATILE=	$(JOU) $(LOG) $(RES) $(KILL) $(ARCHIVES) core
SCRIPTS =	$(MAKEFILE) $(CONFIG) $(RUN) $(AWK)
FILES	=	$(BANNER) $(README) $(SCRIPTS) $(SOURCES) $(DATA)

conf:
		@chmod u+x $(CONFIG)
		@./$(CONFIG)
run:	
		@./$(RUN)
sizes:
		@ls -l $(TARGETS) | tee -a $(SSBASIZE)
clean:
		@rm -f $(VOLATILE) $(NUL)
		@rm -rf test?
readme:
		@$(PAGER) $(README)
print:
		@pr $(PRINTOPT) -h $(NAME) $(SCRIPTS) | $(PRINTER)
printall:
		@pr $(PRINTOPT) -h $(NAME) $(FILES) | $(PRINTER)
tar:
		@tar cf $(TAR) $(FILES)
shar:
		@shar $(FILES) > $(SHAR)
force:

$(BANNER):
		@banner $(NAME) >$@

bench:
		@mkdir testfs1 testfs1/include testfs1/include/sys testfs1/include/netinet
		@mkdir testfs2 testfs2/include testfs2/include/sys testfs2/include/netinet
		@mkdir testfs3 testfs3/include testfs3/include/sys testfs3/include/netinet
		@mkdir testfs4 testfs4/include testfs4/include/sys testfs4/include/netinet
		@mkdir testfs5 testfs5/include testfs5/include/sys testfs5/include/netinet
		@cp ../data/DrawString.c testfs1/DrawString.c
		@cp ../data/MakeLoadable.c testfs1/MakeLoadable.c
		@cp ../data/ProgramName.c testfs1/ProgramName.c
		@cp ../data/RasterFile.h testfs1/RasterFile.h
		@cp ../data/UserDrawS.c testfs1/UserDrawS.c
		@cp ../data/arbitrator.c testfs1/arbitrator.c
		@cp ../data/bres.s testfs1/bres.s
		@cp ../data/cmr.c testfs1/cmr.c
		@cp ../data/cvtalto.c testfs1/cvtalto.c
		@cp ../data/cvtfont.c testfs1/cvtfont.c
		@cp ../data/cvtv1v2.c testfs1/cvtv1v2.c
		@cp ../data/display.h testfs1/display.h
		@cp ../data/font.c testfs1/font.c
		@cp ../data/font.h testfs1/font.h
		@cp ../data/font.v0.h testfs1/font.v0.h
		@cp ../data/font.v1.h testfs1/font.v1.h
		@cp ../data/fontmanip.c testfs1/fontmanip.c
		@cp ../data/fontmanip.h testfs1/fontmanip.h
		@cp ../data/fontnaming.c testfs1/fontnaming.c
		@cp ../data/framebuf.h testfs1/framebuf.h
		@cp ../data/gacha12.c testfs1/gacha12.c
		@cp ../data/graphicops.c testfs1/graphicops.c
		@cp ../data/gxfind.c testfs1/gxfind.c
		@cp ../data/keymap.h testfs1/keymap.h
		@cp ../data/logo.c testfs1/logo.c
		@cp ../data/makefile testfs1/makefile
		@cp ../data/menu.c testfs1/menu.c
		@cp ../data/menu.h testfs1/menu.h
		@cp ../data/mkfont.c testfs1/mkfont.c
		@cp ../data/mksail7.c testfs1/mksail7.c
		@cp ../data/mouse.c testfs1/mouse.c
		@cp ../data/mousedd.c testfs1/mousedd.c
		@cp ../data/profile.c testfs1/profile.c
		@cp ../data/putenv.c testfs1/putenv.c
		@cp ../data/rasterop.c testfs1/rasterop.c
		@cp ../data/sail7.c testfs1/sail7.c
		@cp ../data/sun1bw.c testfs1/sun1bw.c
		@cp ../data/sun1color.c testfs1/sun1color.c
		@cp ../data/sunbitmap.c testfs1/sunbitmap.c
		@cp ../data/suncolor.c testfs1/suncolor.c
		@cp ../data/test.c testfs1/test.c
		@cp ../data/timetest.c testfs1/timetest.c
		@cp ../data/towindow.c testfs1/towindow.c
		@cp ../data/usergraphics.c testfs1/usergraphics.c
		@cp ../data/usergraphics.h testfs1/usergraphics.h
		@cp ../data/util.h testfs1/util.h
		@cp ../data/vec.c testfs1/vec.c
		@cp ../data/window.c testfs1/window.c
		@cp ../data/window.h testfs1/window.h
		@cp ../data/windowops.c testfs1/windowops.c
		@cp ../data/wm.c testfs1/wm.c
		@cp ../data/include/assert.h testfs1/include/assert.h
		@cp ../data/include/colorbuf.h testfs1/include/colorbuf.h
		@cp ../data/include/ctype.h testfs1/include/ctype.h
		@cp ../data/include/errno.h testfs1/include/errno.h
		@cp ../data/include/fcntl.h testfs1/include/fcntl.h
		@cp ../data/include/netdb.h testfs1/include/netdb.h
		@cp ../data/include/sgtty.h testfs1/include/sgtty.h
		@cp ../data/include/signal.h testfs1/include/signal.h
		@cp ../data/include/stdio.h testfs1/include/stdio.h
		@cp ../data/include/time.h testfs1/include/time.h
		@cp ../data/include/netinet/in.h testfs1/include/netinet/in.h
		@cp ../data/include/sys/dir.h testfs1/include/sys/dir.h
		@cp ../data/include/sys/ioctl.h testfs1/include/sys/ioctl.h
		@cp ../data/include/sys/socket.h testfs1/include/sys/socket.h
		@cp ../data/include/sys/stat.h testfs1/include/sys/stat.h
		@cp ../data/include/sys/ttychars.h testfs1/include/sys/ttychars.h
		@cp ../data/include/sys/ttydev.h testfs1/include/sys/ttydev.h
		@cp ../data/include/sys/types.h testfs1/include/sys/types.h
		@cp ../data/include/sys/wait.h testfs1/include/sys/wait.h
		@find .  -print -exec ls -l {} \; 
		@du -s *
		@find .  -print -exec touch {} \;
		@find . -type f -exec grep kangaroo {} \;
		@find . -exec wc {} \;
