#				SSBA 2.0F
#
# Makefile : columbia.mk
#
##############################################################################
SHELL=/bin/sh

.PRECIOUS:	run
.IGNORE:	

PRINTER	=	lp
PRINTOPT=	-l66
PAGER	=	more
COMPILER=	cc
LOCALOPT=	-O
FLOPT=		
SYSTEM	=	-DVMUNIX -DBSD4v3 -DBSD4v2

LABEL	=	

CFLAGS	=	$(LOCALOPT) $(FLOPT)
CC	=	$(COMPILER) $(CFLAGS)

NAME	=	columbia
README	=	${NAME}.doc ${NAME}.files
MAKEFILE=	${NAME}.mk
CONFIG	=	${NAME}.cf
RUN	=	${NAME}.run
JOU	=	${NAME}.jou
LOG	=	${NAME}.log
RES	=	${NAME}.res
KILL	=	${NAME}.kill
TAR	=	${NAME}.tar
AWK	=	${NAME}.awk
SHAR 	=	${NAME}.shar
BANNER 	=	${NAME}.ban
SSBALOG	=	../results/ssba.log
SSBASIZE=	../results/ssba.size
TRACE	=	1>>$(JOU) 2>&1
NUL	=	1>/dev/null 2>&1
SOURCES	=	test1.c test2.c test3.c test4.c test5.c test6.c\
		test7.c test8.c
OBJECTS	=	test1.o test2.o test3.o test4.o test5.o test6.o\
		test7.o test8.o
TARGETS	=	test1 test2 test3 test4 test5 test6 test7 test8
ARCHIVES=	$(TAR) $(SHAR)
VOLATILE=	$(JOU) $(LOG) $(RES) $(KILL) $(OBJECTS) $(TARGETS)\
		$(ARCHIVES) core file
SCRIPTS =	$(MAKEFILE) $(CONFIG) $(RUN) $(AWK)
FILES	=	$(BANNER) $(README) $(SCRIPTS) $(SOURCES)

conf:
		@chmod u+x $(CONFIG)
		@./$(CONFIG)

compile:	$(TARGETS)

run:
		@./$(RUN)
sizes:
		@ls -l $(TARGETS) | tee -a $(SSBASIZE)
clean:
		@rm -f $(VOLATILE) $(NUL)
readme:
		@$(PAGER) $(README)
print:
		@pr $(PRINTOPT) -h $(NAME) $(SCRIPTS) | $(PRINTER)
printall:
		@pr $(PRINTOPT) -h $(NAME) $(FILES) | $(PRINTER)
tar:
		@tar cf $(TAR) $(FILES)
shar:
		@shar $(FILES) > $(SHAR)
force:

$(BANNER):
		@banner $(NAME) >$@

test1:		test1.c
	   	$(CC) $@.c -o $@		$(TRACE)
test2:		test2.c
	   	$(CC) $@.c -o $@		$(TRACE)
test3:		test3.c
	   	$(CC) $@.c -o $@		$(TRACE)
test4:		test4.c
	   	$(CC) $@.c -o $@		$(TRACE)
test5:		test5.c
	   	$(CC) $@.c -o $@		$(TRACE)
test6:		test6.c
	   	$(CC) $@.c -o $@		$(TRACE)
test7:		test7.c
	   	$(CC) $@.c -o $@		$(TRACE)
test8:		test8.c
	   	$(CC) $@.c -o $@		$(TRACE)
