#				SSBA 2.0F
#
# Makefile : config.mk
#
############################################################################
SHELL=/bin/sh

.PRECIOUS:	conf
.PRECIOUS:	run
.IGNORE:	

#
# Options parametrables	(../ssba/ssba.def)
#
PRINTER	=	lp
PRINTOPT=	-l66
PAGER	=	more
COMPILER=	cc
LOCALOPT=	-O
FLOPT=		

#
# Options calculees automatiquement (../ssba/unix.sh, config.cf)
#
SYSTEM	=	-DVMUNIX -DBSD4v3 -DBSD4v2


#
# Commande de compilation resultante
#
CC	=	$(COMPILER)

#
# Fichiers necessaires
#
NAME	=	config
README	=	${NAME}.doc ${NAME}.files
MAKEFILE=	${NAME}.mk
CONFIG	=	${NAME}.cf
RUN	=	${NAME}.run
JOU	=	${NAME}.jou
OUT	=	${NAME}.out
ERR	=	${NAME}.err
TAR	=	${NAME}.tar
SHAR	=	${NAME}.shar
BANNER	=	${NAME}.ban
LOG	=	${NAME}.log
RES	=	${NAME}.res
KILL	=	${NAME}.kill
SSBALOG	=	../results/ssba.log
SSBASIZE=	../results/ssba.size
HEADERS	=	hz.h
DATAS	=	HZ NPROC NFILE
SOURCES	=	chrono.c etime.c hz.c enquire.c nproc.c limit.c
OBJECTS	=	chrono.o etime.o hz.o enquire.o nproc.o limit.o
TARGETS	=	chrono   etime.o hz   enquire   nproc limit
TRACE	=	1>>$(JOU) 2>&1
NUL	=	1>/dev/null 2>&1
SCRIPTS	=	$(MAKEFILE) $(CONFIG) $(RUN)
ARCHIVES=	$(TAR) $(SHAR)
VOLATILE=	$(JOU) $(LOG) $(RES) $(KILL) $(OBJECTS) $(TARGETS)\
		$(DATAS) $(HEADERS) $(ARCHIVES) core
FILES	=	$(BANNER) $(README) $(SCRIPTS) $(HEADERS) $(SOURCES)

conf:
		@chmod u+x $(CONFIG)
		@./$(CONFIG)

compile:	$(TARGETS)

run:
		@./$(RUN)
sizes:
		@ls -l $(TARGETS) | tee -a $(SSBASIZE)
clean:
		@rm -f $(VOLATILE)	$(NUL)
readme:
		@$(PAGER) $(README)
print:
		@pr $(PRINTOPT) -h $(NAME) $(SCRIPTS) | $(PRINTER)
printall:
		@pr $(PRINTOPT) -h $(NAME) $(FILES) | $(PRINTER)
tar:
		@tar cf $(TAR) $(FILES)
shar:
		@shar $(FILES) > $(SHAR)
force:

$(BANNER):
		@banner $(NAME) >$@

chrono:		chrono.c hz.h
		$(CC) $@.c -o $@	$(TRACE)
hz:		hz.c
		$(CC) $(SYSTEM) $@.c -o $@	$(TRACE)
enquire:	enquire.c
		$(CC) -DNO_SC $@.c -o $@	$(TRACE)
nproc:		nproc.c
		$(CC) $@.c -o $@	$(TRACE)
limit:		limit.c
		$(CC) $@.c -o $@	$(TRACE)
etime.o:	etime.c hz.h
		$(CC) -c etime.c	$(TRACE)
