#				SSBA 2.0F
#
# Makefile : filesys.mk
#
##############################################################################
SHELL=/bin/sh

.PRECIOUS:	run
.IGNORE:	

PRINTER	=	lp
PRINTOPT=	-l66
PAGER	=	more
COMPILER=	cc
LOCALOPT=	-O
FLOPT=		
SYSTEM	=	-DVMUNIX -DBSD4v3 -DBSD4v2

LABEL	=	

CFLAGS	=	$(LOCALOPT) $(FLOPT)
CC	=	$(COMPILER) $(CFLAGS)

NAME	=	filesys
README	=	${NAME}.doc ${NAME}.files
MAKEFILE=	${NAME}.mk
CONFIG	=	${NAME}.cf
RUN	=	${NAME}.run
JOU	=	${NAME}.jou
LOG	=	${NAME}.log
RES	=	${NAME}.res
KILL	=	${NAME}.kill
TAR	=	${NAME}.tar
SHAR 	=	${NAME}.shar
BANNER 	=	${NAME}.ban
SSBALOG	=	../results/ssba.log
SSBASIZE=	../results/ssba.size
TRACE	=	1>>$(JOU) 2>&1
NUL	=	1>/dev/null 2>&1
SOURCES	=	prealloc.c randwriter.c randreader.c seqwriter.c seqreader.c\
		routine.c
OBJECTS	=	prealloc.o randwriter.o randreader.o seqwriter.o seqreader.o\
		routine.o
TARGETS	=	prealloc randwriter randreader seqwriter seqreader
ARCHIVES=	$(TAR) $(SHAR)
VOLATILE=	$(JOU) $(LOG) $(RES) $(KILL) $(OBJECTS) $(TARGETS)\
		$(ARCHIVES) core file
SCRIPTS =	$(MAKEFILE) $(CONFIG) $(RUN) $(NAME)
FILES	=	$(BANNER) $(README) $(SCRIPTS) $(SOURCES)

conf:
		@chmod u+x $(CONFIG)
		@./$(CONFIG)

compile:	$(TARGETS)

run:
		@./$(RUN)
sizes:
		@ls -l $(TARGETS) | tee -a $(SSBASIZE)
clean:
		@rm -f $(VOLATILE) $(NUL)
readme:
		@$(PAGER) $(README)
print:
		@pr $(PRINTOPT) -h $(NAME) $(SCRIPTS) | $(PRINTER)
printall:
		@pr $(PRINTOPT) -h $(NAME) $(FILES) | $(PRINTER)
tar:
		@tar cf $(TAR) $(FILES)
shar:
		@shar $(FILES) > $(SHAR)
force:

$(BANNER):
		@banner $(NAME) >$@

prealloc:	prealloc.c routine.o
	   	$(CC) $@.c routine.o -o $@		$(TRACE)
randwriter:	randwriter.c routine.o
	   	$(CC) $@.c routine.o -o $@		$(TRACE)
randreader:	randreader.c routine.o
	   	$(CC) $@.c routine.o -o $@		$(TRACE)
seqwriter:	seqwriter.c routine.o
	   	$(CC) $@.c routine.o -o $@		$(TRACE)
seqreader:	seqreader.c routine.o
	   	$(CC) $@.c routine.o -o $@		$(TRACE)
routine.o:	routine.c
	   	$(CC) $(SYSTEM) -c routine.c 	$(TRACE)
