#include <stdio.h>
#include <fcntl.h>
#include <signal.h>

extern int srand48();
extern long time();
extern double drand48();
extern char *malloc();

int timed_out();

long badreads, filesize, bufsize, reads, maxtime, starttime = 0;
int fd, verbose;
char *filename;

main(argc, argv)        /* random file reads */
int argc;
char *argv[];
{
        register long where, limit;
        char *buffer;

        if (argc < 5) {
          printf("Usage: %s filename filesize bufsize seconds [verbose]\n",
            argv[0]);
          exit(1);
        }
        verbose = (argc > 5);

        if ((fd = open(argv[1], O_RDONLY)) < 0)
            error_message(argv[0],"can't open file");

        filesize = howmuch(argv[2]);
        bufsize = howmuch(argv[3]);
        maxtime = howmuch(argv[4]);
        limit = filesize - bufsize;

        if ((buffer = malloc(bufsize)) == (char *)NULL)
            error_message(argv[0],"malloc failed");

        reads = 0;
        badreads = 0;
        if (srand48((long)(getpid() * getppid())) < 0 )
                error_message(argv[0],"couldn't set seed");

        signal(SIGALRM,timed_out);
        starttime = time((long *)0);
        alarm(maxtime);
        for (;;) {
                where = drand48() * limit;
                if (lseek(fd,where,0) != where) 
                        error_message(argv[0],"random seek failed");
                if (read(fd,buffer,bufsize) != bufsize) badreads++;
                reads++;
        }
}
int timed_out()
{
        long bps, rps, stoptime;
         
        close(fd);
        stoptime = time((long *)0);
        rps = 1.0 * reads/(stoptime - starttime);
        bps = 1.0 * bufsize*reads/(stoptime - starttime);
        if (verbose) {
          printf("%ld second alarm reached\n",maxtime);
          printf("%ld timed seconds\n",stoptime - starttime);
          printf("File is %ld bytes\n",filesize);
          printf("%ld-byte buffers were read %ld times\n",bufsize,reads);
          printf("%ld reads per second\n",rps);
          printf("%ld bytes per second\n",bps);
          printf("%ld bad reads\n",badreads);
        }
        else printf("%10ld%10ld%6ld%12ld%10ld%10ld%10ld\n",
                     filesize,bufsize,maxtime,reads,bps,rps,badreads);
        exit(0);
}
