#include <stdio.h>
#include <fcntl.h>
#include <signal.h>

extern long time();
extern char *malloc();

int timed_out();

long badwrites, filesize, bufsize, total_bytes, writes, maxtime, starttime = 0;
int fd, verbose;
char *filename;

main(argc, argv)        /* sequential file writes */
int argc;
char *argv[];
{
        char *buffer;
        register long subtotal_bytes;
        register int result;

        if (argc < 5) {
          printf("Usage: %s filename filesize bufsize seconds [verbose]\n",
            argv[0]);
          exit(1);
        }
        filesize = howmuch(argv[2]);
        bufsize = howmuch(argv[3]);
        maxtime = howmuch(argv[4]);
        verbose = (argc > 5);
        
        if ((buffer = malloc(bufsize)) == (char *)NULL)
            error_message(argv[0],"malloc failed");
        { register int i; for(i=0;i<bufsize;i++) buffer[i] = 'X'; }

        writes = 0;
        badwrites = 0;
        total_bytes = 0;
        if ((fd = open(argv[1], O_WRONLY, 0666)) < 0)
                error_message(argv[0],"can't open file");

        signal(SIGALRM,timed_out);
        starttime = time((long *)0);
        alarm(maxtime);
        for (;;) {
	    if (lseek(fd,0,0))
		error_message(argv[0],"lseek failed");
            subtotal_bytes = 0;
            while (subtotal_bytes < filesize) {
                result = write(fd,buffer,bufsize);
                total_bytes += result;
                subtotal_bytes += result;
                writes++;
                if (result != bufsize) badwrites++;
            }
        }
}
int timed_out()
{
        long bps, wps, stoptime;
         
        close(fd);
        stoptime = time((long *)0);
        wps = 1.0 * writes/(stoptime - starttime);
        bps = 1.0 * total_bytes/(stoptime - starttime);
        if (verbose) {
          printf("%ld second alarm reached\n",maxtime);
          printf("%ld timed seconds\n",stoptime - starttime);
          printf("File is %ld bytes\n",filesize);
          printf("%ld-byte buffers were written %ld times\n",bufsize,writes);
          printf("%ld total bytes\n",total_bytes);
          printf("%ld writes per second\n",wps);
          printf("%ld bytes per second\n",bps);
          printf("%ld bad writes\n",badwrites);
        }
        else printf("%10ld%10ld%6ld%12ld%10ld%10ld%10ld",
                filesize,bufsize,maxtime,total_bytes,bps,writes,badwrites);
        if (total_bytes < filesize)
                if (verbose) printf("WARNING: Entire file not spanned");
                else printf(" !!!");
        printf("\n");
        exit(0);
}
