/* ioincl.h
/* Copyright (c) 1988, Prime Computer, Inc., Natick, MA 01760
/*                     All Rights Reserved
*/

/* Common header file for IO benchmark

   Initial Version: 10 August 1988 by Barry Wolman

          Modified: 27 September 1988 by Barry Wolman
              1. Add fields to results for tracking response times

          Modified:  2 October 1988 by Barry Wolman
              1. Add calibrate field to control

          Modified: 15 February 1989 by Tom Olson
              1. Add n_reads & n_writes parameters
              2. Add tran_sum_time, tran_sumsq_time, tran_max_time

	  Modified: 25 May 1989 by Barry Wolman
              1. Add variants

          Modified: 20 June 1990 by Barry Wolman
              1. Keep UNIX user & system times separate
              2. Add fields for counted runs
              3. Add checkpoining
*/

#define fexists(x) (access((x),0) == 0)
extern int errno;

#define MAX_FILES  20
#define MAX_USERS  1024
#define MAX_RECLEN 8192      /* Should be 16 or greater */

/* The following value is chosen so that when files are initialized
   on UNIX systems there are no all zero pages */

#define INITRECLEN 512

struct rec {
    long      counter;
    char      junk[MAX_RECLEN];
    };

#define NUMBER_OF_CHECKPOINTS 10

struct checkpoint {
    long      delta,counter;
    double    usert,syst;
    };

struct results {
    long      userno,counter,start,finish;
    int       n_files,n_recs,n_users,locks_per_file,
               reclen,duration,lock_sleep_time,param;
    int       opens[MAX_FILES],closes[MAX_FILES],uses[MAX_FILES];
    long      freads[MAX_FILES],fwrites[MAX_FILES];
    double    sum_time[MAX_FILES],sumsq_time[MAX_FILES],worst_time[MAX_FILES];
    double    tran_sum_time,tran_sumsq_time,tran_max_time;
    int       worst_rec[MAX_FILES];
    double    usert,syst,work_cp;
    long      reads,writes;
    long      resp_dist[26];
    struct    checkpoint checkpoint[NUMBER_OF_CHECKPOINTS+1];
    };

struct control {
    int       runid,runtype;
    int       n_files,n_recs,n_users,locks_per_file,
               reclen,duration,lock_sleep_time,param;
    int       start_delay,end_delay,filelen;
    int       n_reads,n_writes,trans;
    int       calibrate;
    int       variant;
    char      *path[MAX_FILES];
    };

#define LOCKING_VARIANT 1
