#				SSBA 2.0F
#
# Makefile : musbus.mk
#
#############################################################################
SHELL=/bin/sh

.PRECIOUS:	run
.IGNORE:	

SHELL	=	/bin/sh
PRINTER	=	lp
PRINTOPT=	-l66
PAGER	=	more
COMPILER=	cc
LOCALOPT=	-O
FLOPT=		
SYSTEM	=	-DVMUNIX -DBSD4v3 -DBSD4v2
WORKDIR	=	Workload

CFLAGS	=	$(LOCALOPT)
CC	=	$(COMPILER) $(CFLAGS)

NAME	=	musbus
README	=	${NAME}.doc ${NAME}.files
MAKEFILE=	${NAME}.mk Workload/Makefile
CONFIG	=	${NAME}.cf
RUN	=	${NAME}.run
SH	=	${NAME}.sh
LOG	=	${NAME}.log
RES	=	${NAME}.res
JOU	=	${NAME}.jou
KILL	=	${NAME}.kill
WORK	=	${NAME}.work
TAR	=	${NAME}.tar
SHAR 	=	${NAME}.shar
SYN 	=	${NAME}.syn
BANNER 	=	${NAME}.ban
HEADERS	=	${NAME}.h
SSBALOG	=	../results/ssba.log
SSBASIZE=	../results/ssba.size
TRACE	=	1>>$(JOU) 2>&1
NUL	=	1>/dev/null 2>&1
OBJ	=	*.o
TMP	=	Tmp/*
AWK	=	chrono.awk
SED	=	check.sed

ARCHIVES=	$(TAR) $(SHAR)
SOURCES	=	mem.c number.c getopt.c ff.c fstime.c syscall.c pipe.c \
		makework.c util.c getwork.c filter.c keyb.c iamalive.c \
		clock.c ttychk.c mkperm.c hanoi.c acker.c makework.h
WORKSRCS=	$(WORKDIR)/*.c $(WORKDIR)/*.dat
WORKDATA=	$(WORKDIR)/*.txt $(WORKDIR)/DESCRIPTION
WORKSCRI=	$(WORKDIR)/Makefile $(WORKDIR)/script.master\
		$(WORKDIR)/cleanstderr
WORKFILE=	$(WORKSRCS) $(WORKDATA) $(WORKSCRI)
OBJECTS	=	mem.o fstime.o syscall.o pipe.o makework.o util.o getwork.o\
		keyb.o iamalive.o clock.o ttychk.o mkperm.o hanoi.o number.o\
		getopt.o ff.o filter.o acker.o
TARGETS	=	hanoi randmem fstime syscall pipe ffssba\
		makework keyb iamalive clock ttychk mkperm acker
MOSTPROGS=	hanoi randmem fstime syscall pipe mkperm acker
TEMPO	=	$(WORK) $(KILL) core
VOLATILE=	$(JOU) $(LOG) $(RES) $(HEADERS) $(OBJECTS) $(TARGETS)\
		$(ARCHIVES) $(TEMPO) $(SYN)
SCRIPTS =	$(MAKEFILE) $(CONFIG) $(RUN) $(SH) $(AWK) $(SED) \
		cleanup mkscript mkscript.out signature ff.test\
		fixPATH $(WORKSCRI)
FILES	=	$(BANNER) $(README) $(SCRIPTS) $(SOURCES) $(WORKFILE)

conf:
		@chmod u+x $(CONFIG)
		@./$(CONFIG)

compile:	$(TARGETS)

run:
		@./$(RUN)
sizes:
		@ls -l $(TARGETS) | tee -a $(SSBASIZE)
clean:
		@rm -f $(VOLATILE) $(NUL)
		@(cd $(WORKDIR); make $@; rm -f script.? script.out) $(NUL)
		@rm -rf Tmp $(NUL)
readme:
		@$(PAGER) $(README)
print:
		@pr $(PRINTOPT) -h $(NAME) $(SCRIPTS) | $(PRINTER)
printall:
		@pr $(PRINTOPT) -h $(NAME) $(FILES) | $(PRINTER)
tar:
		@tar cf ../$(TAR) .
		@mv ../$(TAR) $(TAR)
shar:
		@shar $(FILES) > $(SHAR)

$(BANNER):
		@banner $(NAME) >$@

scripts:	Tmp Tmp/script.1

Tmp:
		@mkdir Tmp

Tmp/script.1:	mkperm
		@cd $(WORKDIR) ; make scripts
		@rm -f Tmp/script.[0-9]* Tmp/script.out
		@cp $(WORKDIR)/script.[0-9]* $(WORKDIR)/script.out Tmp

hanoi:		hanoi.c
		$(CC) hanoi.c -o $@			$(TRACE)
randmem:	mem.c
		$(CC) $(SYSTEM) -Drandom mem.c -o $@	$(TRACE)
fstime:		fstime.c
		$(CC) $(SYSTEM) fstime.c -o $@		$(TRACE)
syscall:	syscall.c
		$(CC) syscall.c -o $@			$(TRACE)
pipe:		pipe.c
		$(CC) pipe.c -o $@			$(TRACE)
acker:		acker.c
		$(CC) acker.c -o $@			$(TRACE)
keyb:		keyb.c
		$(CC) keyb.c -o $@			$(TRACE)
iamalive:	iamalive.c
		$(CC) iamalive.c -o $@			$(TRACE)
clock:		clock.c
		$(CC) $(SYSTEM) clock.c -o $@		$(TRACE)
ttychk:		ttychk.c
		$(CC) $(SYSTEM) ttychk.c -o $@		$(TRACE)
mkperm:		mkperm.c
		$(CC) mkperm.c -o $@			$(TRACE)
makework:	makework.o util.o getwork.o
		$(COMPILER) makework.o util.o getwork.o -o $@	$(TRACE)
makework.o:	makework.c makework.h
		$(COMPILER) -c makework.c		$(TRACE)
util.o:		util.c makework.h
		$(COMPILER) -c util.c			$(TRACE)
getwork.o:	getwork.c makework.h
		$(COMPILER) -c getwork.c		$(TRACE)
ffssba:		getopt.o filter.o number.o ff.o
		$(COMPILER) getopt.o filter.o number.o ff.o -o $@	$(TRACE)
getopt.o:	getopt.c
		$(COMPILER) -c getopt.c			$(TRACE)
filter.o:	filter.c
		$(COMPILER) -c filter.c			$(TRACE)
number.o:	number.c
		$(COMPILER) -c number.c			$(TRACE)
ff.o:		ff.c
		$(COMPILER) -c ff.c			$(TRACE)
