#				SSBA 2.0F
#
# Makefile : utah.mk
#
##############################################################################
SHELL=/bin/sh

.PRECIOUS:	run
.IGNORE:	

PRINTER	=	lp
PRINTOPT=	-l66
PAGER	=	more
COMPILER=	cc
LOCALOPT=	-O
FLOPT=		
F77CMD=		f77 -O 

CFLAGS	=	$(LOCALOPT) $(FLOPT)
CC	=	$(COMPILER) $(CFLAGS)

NAME	=	utah
README	=	${NAME}.doc ${NAME}.files
MAKEFILE=	${NAME}.mk
CONFIG	=	${NAME}.cf
RUN	=	${NAME}.run
JOU	=	${NAME}.jou
LOG	=	${NAME}.log
RES	=	${NAME}.res
KILL	=	${NAME}.kill
TAR	=	${NAME}.tar
AWK	=	${NAME}.awk
SHAR 	=	${NAME}.shar
BANNER 	=	${NAME}.ban
DATAS	=	${NAME}.data
SSBALOG	=	../results/ssba.log
SSBASIZE=	../results/ssba.size
TRACE	=	1>>$(JOU) 2>&1
NUL	=	1>/dev/null 2>&1
SOURCES	=	bct.sh bytesort.c fibo30.c float.c iocall.c iofile.c\
		savage.c fsavage.f sieve.c
OBJECTS	=	bytesort.o fibo30.o float.o iocall.o iofile.o\
		savage.o fsavage.o sieve.o
TARGETS	=	bytesort   fibo30   float   iocall   iofile  \
		savage fsavage sieve
ARCHIVES=	$(TAR) $(SHAR)
VOLATILE=	$(JOU) $(LOG) $(RES) $(KILL) $(OBJECTS) $(TARGETS)\
		$(ARCHIVES) $(DATAS) core
SCRIPTS =	$(MAKEFILE) $(CONFIG) $(RUN) $(AWK)
FILES	=	$(BANNER) $(README) $(SCRIPTS) $(SOURCES)

conf:
		@chmod u+x $(CONFIG)
		@./$(CONFIG)

compile:	$(TARGETS)

run:	
		@./$(RUN)
sizes:
		@ls -l $(TARGETS) | tee -a $(SSBASIZE)
clean:
		@rm -f $(VOLATILE) $(NUL)
readme:
		@$(PAGER) $(README)
print:
		@pr $(PRINTOPT) -h $(NAME) $(SCRIPTS) | $(PRINTER)
printall:
		@pr $(PRINTOPT) -h $(NAME) $(FILES) | $(PRINTER)
tar:
		@tar cf $(TAR) $(FILES)
shar:
		@shar $(FILES) > $(SHAR)
force:

$(BANNER):
		@banner $(NAME) >$@

bytesort:	bytesort.c
		$(CC) -o $@ bytesort.c	$(TRACE)
savage:		savage.c
		$(CC) -o $@ savage.c -lm	$(TRACE)
fsavage:	fsavage.f
		$(F77CMD) -o $@ fsavage.f	$(TRACE)
fibo30: 	fibo30.c
		$(CC) -o $@ fibo30.c	$(TRACE)
sieve: 		sieve.c
		$(CC) -o $@ sieve.c	$(TRACE)
float: 		float.c
		$(CC) -o $@ float.c	$(TRACE)
iofile: 	iofile.c
		$(CC) -o $@ iofile.c	$(TRACE)
iocall: 	iocall.c
		$(CC) -o $@ iocall.c	$(TRACE)
